#*****************************************************
# Title: Sharpshooter
# Author: Radio Raheem
# Version: 1.1 Beta (11/22/2010)
#
# Sharpshooter Counter-Strike:Source mod inspired by
# Call of Duty: Black Ops.
#
# Description:
#   - Every player uses the same gun
#   - guns randomly cycle every 45 seconds (default).
#   - Unlimited ammo.
#   - Player with the most kills at the end of the
#   round (default: 9 minutes) is the winner.
#
# Installation:
#   - Unzip to "cstrike" directory
#   - Load script once using "es_load sharpshooter"
#   - sharpshooter.cfg will be generated in
#   sharpshooter directory for setting modification.
#*****************************************************

import es
import os
import cfglib
import random
import repeat
import playerlib
import weaponlib
import gamethread
import usermsg

info = es.AddonInfo()
info.name = "Sharpshooter Mode"
info.version = "1.1 Beta"
info.basename = "sharpshooter"
info.url = "http://addons.eventscripts.com/addons/view/sharpshooter"
info.author = "Radio Raheem"

weapons = ['ak47', 'aug', 'awp', 'deagle', 'elite', 'famas',
           'fiveseven', 'g3sg1', 'galil', 'glock', 'hegrenade',
           'knife', 'm249', 'm3', 'm4a1', 'mac10', 'mp5navy', 'p228', 'p90',
           'scout', 'sg550', 'sg552', 'tmp', 'ump45', 'usp', 'xm1014']


config = cfglib.AddonCFG(es.getAddonPath("sharpshooter") + "/sharpshooter.cfg")

config.text("******************************")
config.text("Sharpshooter Configuration")
config.text("   Version 1.0 Beta")
config.text("******************************")

roundtime = config.cvar("ss_roundtime", 9, "Round time in minutes (default = 9).")
weapon_cycle_time = config.cvar("ss_weapon_cycle_time", 45, "Time until weapon change in seconds (default = 45).")
respawn_time = config.cvar("ss_respawn_time", 6, "Wait time for player respawn in seconds (default = 6).")
config.write()


def load():
    config.execute()
    es.doblock('corelib/noisy_on')
    es.set("Sharpshooter_Mode", "Activated")
    es.makepublic("Sharpshooter_Mode")
    es.msg("#lightgreen", "Sharpshooter Mode Loaded!")
    es.log("Sharpshooter Mode Loaded!")
    es.server.queuecmd("mp_ignore_round_win_conditions 1")
    es.server.queuecmd("mp_restartgame 1")
    es.server.cmd("endround")
    weapon_timer = repeat.create('weapon_timer', new_weapon)

def player_activate(event_var):
    SpawnPlayer(int(event_var['userid']))
            
def new_weapon():
    set_weapon()
    PlayerList = playerlib.getPlayerList('#alive')
    for Player in PlayerList:
        remove_player_weapons(Player.userid)
        give_weapon(Player.userid)
    RemoveWeaponsOnGround()
    
def set_weapon():
    RemoveWeaponsOnGround()
    es.set("current_weapon", random.choice(weapons))

def give_weapon(userid):
    remove_player_weapons(userid)
    usermsg.hudhint(userid, "Current weapon: %s" % es.ServerVar("current_weapon"))
    es.delayed(0.1, 'es_xgive %s weapon_%s' % (userid, es.ServerVar("current_weapon")))
    
def round_start(event_var):
    set_weapon()
    PlayerList = playerlib.getPlayerList('#alive')
    es.forcevalue("mp_roundtime", roundtime)
    gamethread.delayedname(roundtime * 60, "roundtime_delay", end_game)
    for buyzone in es.createentitylist('func_buyzone'):
        es.server.queuecmd('es_xremove %s' % buyzone)
    weapon_timer = repeat.find('weapon_timer')
    weapon_timer.start(weapon_cycle_time, 0)

    entities = ('func_hostage_rescue', 'hostage_entity', 'func_bomb_target', 'weapon_c4')
    userid = es.getuserid()
    if userid:
        for entity in entities:
            es.server.queuecmd('es_xfire %s %s Kill' % (userid, entity))
    RemoveWeaponsOnGround()

def round_end(event_var):
    gamethread.cancelDelayed("roundtime_delay")
    weapon_timer = repeat.find('weapon_timer')
    weapon_timer.stop()

def remove_player_weapons(userid):
    userid = int(userid)
    es.server.queuecmd("es_xfire %i player_weaponstrip Kill" % userid)
    es.server.queuecmd("es_xgive %i player_weaponstrip" % userid)
    es.server.queuecmd("es_xfire %i player_weaponstrip Strip" % userid)
    
def RemoveWeaponsOnGround():
    for index in es.createentityindexlist():
        if es.entitygetvalue(index, 'classname')[0:6] != 'weapon':
            continue
        if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == -1:
            es.server.queuecmd('es_xremove %i' % index)
    
def end_game():
    winner = getMostKills()
    winner_name = es.getplayername(winner[0])
    es.msg('#multi', '#lightgreen[Sharpshooter Mode]:#default The Winner is#green %s#default with#green %s#default kills!' % (winner_name, winner[1]))
    es.centermsg("The Winner is %s!" % winner_name)
    weapon_timer = repeat.find('weapon_timer')
    weapon_timer.stop()
    EndGame(winner[0])
    
def EndGame(userid):
    es.server.queuecmd("es_xgive %s game_end" % userid)
    es.server.queuecmd("es_xfire %s game_end EndGame" % userid)
    es.server.queuecmd("mp_restartgame 2")
    
def player_spawn(event_var):
    remove_player_weapons(event_var['userid'])
    give_weapon(event_var['userid'])

def player_team(event_var):
    if event_var['team'] != 1:
        es.server.queuecmd('es_xspawnplayer %s' % event_var['userid'])
    
def player_death(event_var):
    userid = int(event_var['userid'])
    gamethread.delayed(respawn_time, SpawnPlayer, userid)
    if respawn_time != 1:
        usermsg.hudhint(event_var['userid'], "Respawning in %s seconds." % respawn_time)
    else:
        usermsg.hudhint(event_var['userid'], "Respawning in %s second." % respawn_time)

def SpawnPlayer(userid):
    if es.exists('userid', userid) and es.getplayerteam(userid) > 1:
        es.setplayerprop(userid, 'CCSPlayer.m_iPlayerState', 0)
        es.setplayerprop(userid, 'CCSPlayer.baseclass.m_lifeState', 512)
        es.server.queuecmd('es_xspawnplayer %i' % userid)
   
def getMostKills():
    """
    Returns the player with the most kills in a tuple:
    (userid, kill count)
    """
    playerlist = es.createplayerlist()
    if not playerlist:
        return None, None

    userid = max(playerlist, key=lambda userid: playerlist[userid]['kills'])
    return userid, playerlist[userid]['kills']

def weapon_fire_on_empty(event_var):
    es.server.queuecmd('es_xgive %i %s' % (int(event_var['userid']), 'info_null'))

def weapon_reload(event_var):
    es.server.queuecmd('es_xgive %i %s' % (int(event_var['userid']), 'info_null'))

def hegrenade_detonate(event_var):
    userid = event_var['userid']
    if es.ServerVar("current_weapon") == "hegrenade":
        es.server.queuecmd("es_xgive %s weapon_hegrenade" % userid)
        
def unload():
    es.doblock('corelib/noisy_off')
    gamethread.cancelDelayed("roundtime_delay")
    weapon_timer = repeat.find('weapon_timer')
    weapon_timer.stop()
    weapon_timer.delete()
    es.server.queuecmd("mp_ignore_round_win_conditions 0")
    es.set("Sharpshooter_Mode", "Deactivated")
    es.makepublic("Sharpshooter_Mode")
    es.msg("#lightgreen", "Sharpshooter Mode Unloaded!")
    es.log("Sharpshooter Mode Unloaded!")

